;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  Authors
;;;  		Nickolay V. Shmyrev <nshmyrev@yandex.ru>
;;;

;;; Try to find out where we are
(if (assoc 'msu_ru_nsh_diphone voice-locations)
    (defvar msu_ru_nsh_dir 
      (cdr (assoc 'msu_ru_nsh_diphone voice-locations)))
    ;;; Not installed in Festival yet so assume running in place
    (defvar msu_ru_nsh_dir (pwd)))

(if (not (probe_file (path-append msu_ru_nsh_dir "festvox/")))
    (begin
     (format stderr "msu_ru_nsh: Can't find voice scm files they are not in\n")
     (format stderr "   %s\n" (path-append msu_ru_nsh_dir "festvox/"))
     (format stderr "   Either the voice isn't linked into Festival\n")
     (format stderr "   or you are starting festival in the wrong directory\n")
     (error)))

;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons (path-append msu_ru_nsh_dir "festvox/") load-path))

;;; Voice specific parameter are defined in each of the following
;;; files
(require 'msu_ru_nsh_phoneset)
(require 'msu_ru_nsh_tokenizer)
(require 'msu_ru_nsh_tagger)
(require 'msu_ru_nsh_lexicon)
(require 'msu_ru_nsh_phrasing)
(require 'msu_ru_nsh_intonation)
(require 'msu_ru_nsh_duration)
(require 'msu_ru_nsh_f0model)
(require 'msu_ru_nsh_other)
;; ... and others as required

;;;  Ensure we have a festival with the right diphone support compiled in
(require_module 'UniSyn)

(set! msu_ru_nsh_lpc_sep 
      (list
       '(name "msu_ru_nsh_lpc_sep")
       (list 'index_file (path-append msu_ru_nsh_dir "dic/nshdiph.est"))
       '(grouped "false")
       (list 'coef_dir (path-append msu_ru_nsh_dir "lpc"))
       (list 'sig_dir  (path-append msu_ru_nsh_dir "lpc"))
       '(coef_ext ".lpc")
       '(sig_ext ".res")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

(set! msu_ru_nsh_lpc_group 
      (list
       '(name "nsh_lpc_group")
       (list 'index_file 
	     (path-append msu_ru_nsh_dir "group/nshlpc.group"))
       '(grouped "true")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

;; Go ahead and set up the diphone db
;;(set! msu_ru_nsh_db_name (us_diphone_init msu_ru_nsh_lpc_sep))
;; Once you've built the group file you can comment out the above and
;; uncomment the following.
(set! msu_ru_nsh_db_name (us_diphone_init msu_ru_nsh_lpc_group))

(define (msu_ru_nsh_diphone_fix utt)
"(msu_ru_nsh_diphone_fix UTT)
Map phones to phonological variants if the diphone database supports
them."
  (mapcar
   (lambda (s)
     (let ((name (item.name s)))
       (msu_ru_nsh_diphone_fix_phone_name utt s)
       ))
   (utt.relation.items utt 'Segment))
  utt)

(define (msu_ru_suitable_cluster n1 n2 clust_data)
"(msu_ru_suitable_cluster_name N1 N2 CLUST_DATA)
Search in cluster list and returns t if we have 
such cluster"
    (cond
	((null clust_data) nil)
	((and (string-equal n1 (car (car clust_data)))
	      (member_string n2 (car (cdr (car clust_data)))))
	  t)
	(t (msu_ru_suitable_cluster n1 n2 (cdr clust_data)))
    )
)

(define (msu_ru_nsh_diphone_fix_phone_name utt seg)
"(msu_ru_nsh_fix_phone_name UTT SEG)
Add the feature diphone_phone_name to given segment with the appropriate
name for constructing a diphone.  Basically adds _ if either side is part
of the same consonant cluster, adds $ either side if in different
syllable for preceding/succeeding vowel syllable."
  (let ((name (item.name seg)))

;;  (format stderr "Post processing diphone for %s\n" name)
;;  (format stderr "%s\n" name)

    (cond
     ((string-equal name "#") t)
     ((string-equal "-" (item.feat seg 'ph_vc))
      (if (and (string-equal "-" (item.feat (item.prev seg) 'ph_vc))
	       (item.relation.prev seg "SylStructure")
	       (msu_ru_suitable_cluster (item.feat seg "p.name") name msu_ru::clust-data))
	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
      (if (and (string-equal "-" (item.feat (item.next seg) 'ph_vc))
	       (item.relation.next seg "SylStructure")
	       (msu_ru_suitable_cluster name (item.feat seg "n.name") msu_ru::clust-data))
	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))))
   ))

(define (msu_ru_nsh_voice_reset)
  "(msu_ru_nsh_voice_reset)
Reset global variables back to previous voice."
  (msu_ru_nsh::reset_phoneset)
  (msu_ru_nsh::reset_tokenizer)
  (msu_ru_nsh::reset_tagger)
  (msu_ru_nsh::reset_lexicon)
  (msu_ru_nsh::reset_phrasing)
  (msu_ru_nsh::reset_intonation)
  (msu_ru_nsh::reset_duration)
  (msu_ru_nsh::reset_f0model)
  (msu_ru_nsh::reset_other)
)

;;;  Full voice definition 
(define (voice_msu_ru_nsh_diphone)
"(voice_msu_ru_nsh_diphone)
Set speaker to nsh in us from msu."
  ;; Select appropriate phone set
  (msu_ru_nsh::select_phoneset)

  ;; Select appropriate tokenization
  (msu_ru_nsh::select_tokenizer)

  ;; For part of speech tagging
  (msu_ru_nsh::select_tagger)

  (msu_ru_nsh::select_lexicon)

  (msu_ru_nsh::select_phrasing)

  (msu_ru_nsh::select_intonation)

  (msu_ru_nsh::select_duration)

  (msu_ru_nsh::select_f0model)

  ;; Waveform synthesizer: UniSyn diphones
  (set! UniSyn_module_hooks (list msu_ru_nsh_diphone_fix))
  (set! us_abs_offset 0.0)
  (set! window_factor 1.0)
  (set! us_rel_offset 0.0)
  (set! us_gain 0.9)

  (Parameter.set 'Synth_Method 'UniSyn)
  (Parameter.set 'us_sigpr 'lpc)
  (us_db_select msu_ru_nsh_db_name)

  ;; This is where you can modify power (and sampling rate) if desired
  (set! after_synth_hooks nil)
;  (set! after_synth_hooks
;      (list
;        (lambda (utt)
;          (utt.wave.rescale utt 2.1))))

  ;; set callback to restore some original values changed by this voice
  (set! current_voice_reset msu_ru_nsh_voice_reset)

  (set! current-voice 'msu_ru_nsh_diphone)
)

(proclaim_voice
 'msu_ru_nsh_diphone
 '((language russian)
   (gender male)
   (dialect moscow)
   (description
    "This is implementation of russian voice for festival"
    )
   (builtwith festvox-1.3)))

(provide 'msu_ru_nsh_diphone)
