;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Lexicon, LTS and Postlexical rules for msu_ru
;;;

;;; Load any necessary files here

(define (msu_ru_addenda)
  "(msu_ru_addenda)
Basic lexicon should (must ?) have basic letters, symbols and punctuation."

(lex.add.entry '("" n ( ((sh t o) 0) )))
(lex.add.entry '("" n ( ((sh t o) 1) ((b i) 0) )))
(lex.add.entry '("-" n ( ((k o) 0)  ((ij e) 0) ((sh t o) 1) )))
(lex.add.entry '("-" n ( ((sh t o) 1) ((nn i) 0) ((b u dd) 0) )))

;;; Symbols ...
(lex.add.entry 
 '("*" n ( ((u) 0) ((m n o) 1) ((zh i tt) 0) )))
(lex.add.entry 
 '("%" n ( ((p r o) 1) ((c e n t) 0) )))

;; Basic punctuation must be in with nil pronunciation
(lex.add.entry '("." punc nil))
(lex.add.entry '("." nn (((t o) 1) ((ch k a) 0))))
(lex.add.entry '("'" punc nil))
(lex.add.entry '(":" punc nil))
(lex.add.entry '(";" punc nil))
(lex.add.entry '("," punc nil))
(lex.add.entry '("-" punc nil))
(lex.add.entry '("\"" punc nil))
(lex.add.entry '("`" punc nil))
(lex.add.entry '("?" punc nil))
(lex.add.entry '("!" punc nil))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Hand written letter to sound rules
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(lts.ruleset
 russian_downcase
 ( )
 (
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  ) 
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )

  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  ) 
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )
  ( [  ] =  )

  ( [ - ] = - )
  
  ( [ 1 ] = )
  ( [ 2 ] = )
  ( [ 3 ] = )
  ( [ 4 ] = )
  ( [ 5 ] = )
  ( [ 6 ] = )
  ( [ 7 ] = )
  ( [ 8 ] = )
  ( [ 9 ] = )
  ( [ 0 ] = )
))


(define (russian_downcase word)
  "(russian_downcase WORD)
Downs case word by letter to sound rules since builtin downcase is only ascii."
  (lts.apply word 'russian_downcase))



(define (russian_contains_vowel phones)
  (cond
   ((null phones) nil)
   ((member_string (car phones) '(a e i o u))
    t)
   (t (russian_contains_vowel (cdr phones)))))


(define (russian_syllabify_phstress phones syls)
"(russian_syllabification phones syls)
Syllabify phones into syllables for Russian."
 (cond
  ((null phones)
   (mapcar
    (lambda (syl) (list (reverse syl) 0))
    (reverse syls)))
  ((and (member_string (car phones) '(a e i o u))
	(russian_contains_vowel (cdr phones)))
   (let ((nsyls (cons (cons (car phones) (car syls)) (cdr syls)))
	 (nphones (cdr phones)))
     (cond
      ((and (member_string (car nphones) '(l ll m mm n nn r rr))
	    (not (member_string (car (cdr nphones)) 
				'(a e i o u l ll m mm n nn r rr))))
       (set! nsyls (cons (cons (car nphones) (car nsyls)) (cdr nsyls)))
       (set! nphones (cdr nphones)))
      ((and (string-equal (car nphones) "ij")
	    (not (member_string (car (cdr nphones)) 
				'(a e i o u))))
       (set! nsyls (cons (cons (car nphones) (car nsyls)) (cdr nsyls)))
       (set! nphones (cdr nphones)))
       )
     (russian_syllabify_phstress nphones (cons nil nsyls))))
  (t
   (russian_syllabify_phstress
    (cdr phones)
    (cons (cons (car phones) (car syls)) (cdr syls))))))


;;;  Function called when word not found in lexicon
(define (msu_ru_lts_function word features)
  "(msu_ru_lts_function WORD FEATURES)
Return pronunciation of word not in lexicon."
;;  (format stderr "failed to find pronunciation for %s\n" word)
  (let (phones syls letters)

    (set! letters (lts.apply word 'russian_downcase))
    (set! phones (lts.apply letters 'msu_ru))

    ;;; Need to assign lexical stress
    (set! syls (russian_syllabify_phstress phones nil))
    ;;; Test syllabify (format stderr "%l\n" syls)

    (list word features syls)
    ))


;; You may or may not be able to write a letter to sound rule set for
;; your language.  If its largely lexicon based learning a rule
;; set will be better and easier that writing one (probably).
(lts.ruleset
 msu_ru
   (  
     (SOFTLETTERS 
	 ;;; Letter that do make previous consonant soft
                  
         ;;; Some softization that I like to have
	          
     )
     (STARTSYL #           )
   )
 (
;; LTS rules

;; Non readable phonems
 
    ( [   ]  = s )
    ( [    ]  = s )
    ( [   ]  = r )
    ( [   ]  = r )
    ( [   ]  = n )
    ( [   ]  = s )
    ( [   ]  = z )
    ( [   ]  = n )
    ( [   ]  = n )
    ( [   ]  = s )    
    ( [   ]  = z )    
    ( [   ]  = n )    
    ( [   ]  = n )    
    ( [   ]  = n )    
    
;; Work around doubled consonants 

    ( [   ] # = n )
    ( [   ] # = s )
    ( [   ] # = m )
    
;; My lovely proj. pronouncation    

    ( [    ] # = k i ij )
    ( [    ] # = g i ij )
    ( [    ] # = h i ij )
    ( [    ] # = o v o )
    ( [    ] # = e v o )

;; Simple vowels

    ( [  ] = a )
    ( [  ] = i )
    ( [  ] = o )
    ( [  ] = u )
    ( [  ] = e )

;; Vovel expansion

    ( STARTSYL [  ] = ij u )
    ( STARTSYL [  ] = ij a )
    ( STARTSYL [  ] = ij e )
    ( STARTSYL [  ] = ij o )

    ( [  ] = o )
    ( [  ] = e )
    ( [  ] = a )
    ( [  ] = u )
    ( [  ] = i )

;; Soft 

    ( [  ] SOFTLETTERS = bb )
    ( [  ] SOFTLETTERS = vv )
    ( [  ] SOFTLETTERS = gg )
    ( [  ] SOFTLETTERS = dd )
    ( [  ] SOFTLETTERS = zhz )
    ( [  ] SOFTLETTERS = zz )
    ( [  ] SOFTLETTERS = kk )
    ( [  ] SOFTLETTERS = ll )
    ( [  ] SOFTLETTERS = mm )
    ( [  ] SOFTLETTERS = nn )
    ( [  ] SOFTLETTERS = pp )
    ( [  ] SOFTLETTERS = rr )
    ( [  ] SOFTLETTERS = ss )
    ( [  ] SOFTLETTERS = tt )
    ( [  ] SOFTLETTERS = ff )
    ( [  ] SOFTLETTERS = hh )

;; Simple consonant

    ( [  ] = b )
    ( [  ] = v )
    ( [  ] = g )
    ( [  ] = d )
    ( [  ] = zh )
    ( [  ] = z )
    ( [  ] = k )
    ( [  ] = l )
    ( [  ] = m )
    ( [  ] = n )
    ( [  ] = p )
    ( [  ] = r )
    ( [  ] = s )
    ( [  ] = t )
    ( [  ] = f )
    ( [  ] = h )
    ( [  ] = c )
    ( [  ] = ch )
    ( [  ] = sh )
    ( [  ] = sch )
    ( [  ] = ij )

    ( [  ] =  )
    ( [  ] =  )
    ( [ - ]  = )
  ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Postlexical Rules 
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (msu_ru::postlex_rule1 utt)
  "(msu_ru::postlex_rule1 utt)
   Modification of consonants voiced/voiceless"

  (mapcar
   (lambda (s)

     (set! s1 (item.next s))

     (if (and (string-equal "1" (item.feat s "syl_final"))
	      (not (string-equal "0" 
				 (item.feat s "R:SylStructure.parent.syl_break"))))
	 (mapcar 
	  (lambda (r)
	    (if (string-equal (car r) (item.name s)) 
		(if (or 
		     (string-equal "+" (item.feat s1 "ph_vc"))
		     (member_string (item.name s1) msu_ru::sonorlist) 
		     (string-equal "#" (item.name s1)) 
		
		     (and 
		      (or (string-equal "v" (item.name s1)) 
			  (string-equal "vv" (item.name s1)))
			
		      (or (string-equal "+" (item.feat s1 "n.ph_vc"))
			  (member_string (item.feat s1 "n.name") msu_ru::sonorlist)))
		    )
		    (item.set_name s (car (cdr r))))

	      (if (string-equal (car (cdr r)) (item.name s)) 
		  (if 
		     (and 
		      (or (string-equal "v" (item.name s1)) 
			  (string-equal "vv" (item.name s1)))
			      (member_string (item.feat s1 "n.name") msu_ru::voicelist))
		      (item.set_name s (car r))))
		      ))
	  msu_ru::reducelist))
	  
	  (mapcar 
		  (lambda (r)
    		    (if (and (string-equal (car r) (item.name s))
			     (member_string (item.name s1) msu_ru::voicelesslist))
			     (item.set_name s (car (cdr r))))
	    
		    (if (and (string-equal (car (cdr r)) (item.name s)) 
			     (member_string (item.name s1) msu_ru::voicelist))
			      (item.set_name s (car r))))
		   msu_ru::reducelist))
   
   (utt.relation.items utt 'Segment)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Lexicon definition
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(lex.create "msu_ru")
(lex.set.phoneset "msu_ru")
(lex.set.lts.method 'msu_ru_lts_function)
;;; If you have a compiled lexicon uncomment this
;(lex.set.compile.file (path-append msu_ru_nsh_dir "festvox/msu_ru_lex.out"))
(msu_ru_addenda)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Lexicon setup
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (msu_ru_nsh::select_lexicon)
  "(msu_ru_nsh::select_lexicon)
Set up the lexicon for msu_ru."
  (lex.select "msu_ru")

  ;; Post lexical rules
  (set! postlex_rules_hooks (list msu_ru::postlex_rule1))
)

(define (msu_ru_nsh::reset_lexicon)
  "(msu_ru_nsh::reset_lexicon)
Reset lexicon information."
  t
)

(provide 'msu_ru_nsh_lexicon)
