;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Phonset for msu_ru
;;;

(defPhoneSet
  msu_ru
  ;;;  Phone Features
  (;; vowel or consonant
   (vc + -)  
   ;; vowel length: short long dipthong schwa
   (vlng s l d a 0)
   ;; vowel height: high mid low
   (vheight 1 2 3 0)
   ;; vowel frontness: front mid back
   (vfront 1 2 3 0)
   ;; lip rounding
   (vrnd + - 0)
   ;; consonant type: stop fricative affricative nasal liquid
   (ctype s f a n l 0)
   ;; place of articulation: labial alveolar palatal labio-dental
   ;;                         dental velar
   (cplace l a p b d v 0)
   ;; consonant voicing
   (cvox + - 0)
   )
  (
   (#  - 0 0 0 0 0 0 -)  ;; slience ... 

   ;;; vowels
   
   (i       +  l  1  1  0  0  0  -)
   (u       +  l  1  3  -  0  0  -)
   (e       +  l  2  2  0  0  0  -)
   (o       +  l  2  2  -  0  0  -)
   (a       +  l  3  2  0  0  0  -)

   ;;; consonants

   (p       -  0  0  0  0  s  l  -)
   (pp      -  0  0  0  0  s  l  -)
   (b       -  0  0  0  0  s  l  +)
   (bb      -  0  0  0  0  s  l  +)
   (f       -  0  0  0  0  f  b  -)
   (ff      -  0  0  0  0  f  b  -)
   (v       -  0  0  0  0  f  b  +)
   (vv      -  0  0  0  0  f  b  +)
   (m       -  0  0  0  0  n  l  +)
   (mm 	    -  0  0  0  0  n  l  +)
   (t 	    -  0  0  0  0  s  d  -)
   (tt 	    -  0  0  0  0  s  d  -)
   (d 	    -  0  0  0  0  s  d  +)
   (dd 	    -  0  0  0  0  s  d  +)
   (c       -  0  0  0  0  a  d  +)
   (s 	    -  0  0  0  0  f  d  -)
   (ss 	    -  0  0  0  0  f  d  -)
   (z 	    -  0  0  0  0  f  d  +)
   (zz 	    -  0  0  0  0  f  d  +)
   (n       -  0  0  0  0  n  d  +)
   (nn 	    -  0  0  0  0  n  d  +)
   (l 	    -  0  0  0  0  l  d  +)
   (ll 	    -  0  0  0  0  l  d  +)
   (sh      -  0  0  0  0  f  a  -)
   (sch     -  0  0  0  0  f  p  -)
   (zh      -  0  0  0  0  f  a  +)
   (zhz     -  0  0  0  0  f  a  +)
   (ch      -  0  0  0  0  a  a  -)
   (r 	    -  0  0  0  0  l  a  +)
   (rr 	    -  0  0  0  0  l  a  +)
   (ij 	    -  0  0  0  0  a  p  +)
   (k 	    -  0  0  0  0  s  p  -)
   (kk      -  0  0  0  0  s  p  -)
   (g 	    -  0  0  0  0  s  p  +)
   (gg 	    -  0  0  0  0  s  p  +)
   (h 	    -  0  0  0  0  f  v  -)
   (hh 	    -  0  0  0  0  f  v  -)

  )
)

;;; Some grouping

(set! msu_ru::vowels '(a i u e o))
(set! msu_ru::consonants '(p pp b bb f ff v vv m mm t tt d dd c s ss z zz n nn l ll sh sch zh zhz  ch r rr ij k kk g gg h hh))
(set! msu_ru::reducelist '((b p) (bb pp) (v f) (vv ff) (zh sh) (z s) (zz ss) (g k) (gg kk) (d t) (dd tt)))
(set! msu_ru::sonorlist '(l ll m mm n nn r rr ij))
(set! msu_ru::voicelist '(zh zhz z zz d dd g gg))
(set! msu_ru::voicelesslist '(sh s ss t tt k kk sch))

;;; Statistically collected clusters. Probably it needs to be
;;; theorethized. I am going to include here something like this
;;; (first_phone (list of second phones)). This contains most 
;;; frequent combinations

(set! msu_ru::clust-data '(
			   ( ch ( t tt ij sh n k kk  ))
			   ( zh ( n dd nn d  ))
			   ( sch ( nn  ))
			   ( vv ( ij  ))
			   ( pp ( sch ij  ))
			   ( tt ( sh ch ij ss  ))
			   ( sh ( nn n kk ll k l t  ))
			   ( ff ( sh  ))
			   ( b ( l rr ll ij r n  ))
			   ( d ( c p ll vv rr dd n nn r d v  ))
			   ( g ( dd d r n nn rr l ll  ))
			   ( f ( r k ss rr s t  ))
			   ( ss ( m sh ch  ))
			   ( h ( l v r n  ))
			   ( k ( tt rr n v r k nn ll l s t  ))
			   ( m ( ll n m nn  ))
			   ( l ( ll n nn  ))
			   ( n ( nn t n  ))
			   ( p ( rr ll r s l k n  ))
			   ( s ( r n pp kk l s rr vv t ss mm ll k p dd h v tt nn m  ))
			   ( r ( n nn m  ))
			   ( t ( s vv v ss nn c p rr t ll kk n k r  ))
			   ( v ( nn l mm r d zz pp ll n rr p z  ))
			   ( z ( dd n nn rr b v m d r vv bb mm ll g l z  ))
			   ( ll ( v nn n  ))
			   ))



(PhoneSet.silences '(#))

(define (msu_ru_nsh::select_phoneset)
  "(msu_ru_nsh::select_phoneset)
Set up phone set for msu_ru."
  (Parameter.set 'PhoneSet 'msu_ru)
  (PhoneSet.select 'msu_ru)
)

(define (msu_ru_nsh::reset_phoneset)
  "(msu_ru_nsh::reset_phoneset)
Reset phone set for msu_ru."
  t
)

(provide 'msu_ru_nsh_phoneset)
