;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                       ;;
;;;  A Scheme for a Russian set of diphones                               ;;
;;;  Inspired by Steve Isard's diphone schemas from CSTR, University of   ;;
;;;  Edinburgh                                                            ;;
;;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons "festvox/") load-path)

;;; Voice specific parameter are defined in each of the following
;;; files
(require 'msu_ru_nsh_phoneset)

;;; A diphone list 

;;; When building diphone list we will follow the rules

;;; All vowels are taken

;;; We take all consonant-consonant combination since the word can 
;;; have any phone in the and and any phone at the start. The only
;;; exception that we take into account will be rules for voicing-devoicing
;;; described in lexicon postprocessing

;;; For consonant-consonant clusters we will take statistically colleced
;;; combinations that can be archieved from reading large text like 
;;; Ozhegov's dictionary


(set! vowels msu_ru::vowels)
(set! all_consonants msu_ru::consonants)
(set! clust-data msu_ru::clust-data)
(set! silence '#)

(set! cvc-carrier '((# t a ) (#)))
(set! vv-carrier '((# t a k) (t a #)))

(set! silv-carrier '(() (t a #)))
(set! silc-carrier '(() (t a #)))
(set! vsil-carrier '((# t a k ) ()))
(set! csil-carrier '((# t a t a) ()))

(set! cc-carrier '((# t a ) (a t a #)))
(set! ccclust-carrier '((# t a -) (a t a #)))

;;; These functions simply fill out the nonsense words
;;; from the carriers and vowel consonant definitions

(define (list-cvc)
  (let ((longlist nil))
    (mapcar
     (lambda (v)
       (mapcar
	(lambda (c)
	  (let ((dlist (list (string-append v "-" c) (string-append c "-" v)))
		(rc c) (rv 'a))
	    (set! longlist
		  (cons
		   (list dlist
			 (append (car cvc-carrier) (list rc v c rv) 
				 (car (cdr cvc-carrier))))
		   longlist))))
	all_consonants))
       vowels)
    (reverse longlist)))

(define (list-vv)
  (let ((longlist nil))
    (mapcar
    (lambda (v1)
      (mapcar
       (lambda (v2)
	 (set! longlist
	       (cons
		(list
		 (list (string-append v1 "-" v2))
		 (append (car vv-carrier) (list v1 v2) 
			 (car (cdr vv-carrier))))
		longlist)))
       vowels))
    vowels)
    (reverse longlist)))

(define (list-cc)
  (let ((longlist nil))
    (mapcar
    (lambda (c2)
      (mapcar
       (lambda (c1)
    	   (set! longlist
	       (cons
		(list
		 (list (string-append c1 "-" c2))
		 (append (car cc-carrier) 
			 (list c1 "-" c2) 
			 (car (cdr cc-carrier))))
		longlist))
	)
       (remove-list all_consonants 
        ((lambda (c) 
	    (let ((removelist nil))
		(if (or (member_string c2 msu_ru::voicelist) (member_string c2 msu_ru::voicelesslist))
		(mapcar
		  (lambda (phonepair)
		    (if (member_string c2 msu_ru::voicelesslist)
			  (set! removelist (cons (car phonepair) removelist))
			  (set! removelist (cons (car (cdr phonepair)) removelist))
		    )
		  )
		msu_ru::reducelist))
	    (reverse removelist))
	    )
	 c2)
	)))
    all_consonants)
    (reverse longlist)))

(define (list-cccluster)
  (let ((longlist nil))
    (mapcar
    (lambda (start_data)
      (set! c1 (car start_data))
      (mapcar
       (lambda (c2)
	 (set! longlist
	       (cons
		(list
		 (list (string-append c1 "_-_" c2))
		 (append (car ccclust-carrier) (list c1 c2) 
			 (car (cdr ccclust-carrier))))
		longlist)))
       (car (cdr start_data))))
    clust-data)
    (reverse longlist)))

(define (list-sil)
  (let ((longlist nil))
   (mapcar
    (lambda (v) 
      (set! longlist
	    (cons
	     (list
	      (list (string-append silence "-" v))
	      (append (car silv-carrier) (list silence v) 
		      (car (cdr silv-carrier))))
	     longlist)))
    vowels)
   (mapcar
    (lambda (c) 
      (let ((rv 'a))
	(set! longlist
	    (cons
	     (list
	      (list (string-append silence "-" c))
	      (append (car silc-carrier) (list silence c rv) 
		      (car (cdr silc-carrier))))
	     longlist))))
    all_consonants)
   (mapcar
    (lambda (v) 
      (set! longlist
	    (cons
	     (list
	      (list (string-append v "-" silence))
	      (append (car vsil-carrier) (list v silence) 
		      (car (cdr vsil-carrier))))
	     longlist)))
    vowels)
   (mapcar
    (lambda (c) 
      (set! longlist
	    (cons
	     (list
	      (list (string-append c "-" silence))
	      (append (car csil-carrier) (list c silence) 
		      (car (cdr csil-carrier))))
	     longlist)))
    all_consonants)
   (reverse longlist)))


;;; End of individual generation functions

(define (diphone-gen-list)
  "(diphone-gen-list)
Returns a list of nonsense words as phone strings."
  (append
   (list-cvc)       ;; consonant-vowel-consonant
   (list-vv)        ;; vowel-vowel
   (list-cc)        ;; consonant-consonant on the syllable boundary
   (list-cccluster) ;; consonant clusters in the same syllable
   (list-sil)       ;; into and out of silence
   (list
    '(("#-#") (# t a t a # #)))
   ))

(define (Diphone_Prompt_Setup)
 "(Diphone_Prompt_Setup)
Called before synthesizing the prompt waveforms.  Cross language prompts
from US male (for gaijin male)."
 (voice_kal_diphone)  ;; US male voice
 (set! FP_F0 90)
 (set! diph_do_db_boundaries nil) ;; cross-lang confuses this
 )

(set! russian2radio_map
      '((a aa)
	(i iy)
	(o ow)
	(u uw)
	(e eh)
	(p p)
	(pp p)
	(b b)
	(bb b)
	(f f)
	(ff f)
	(v v)
	(vv v)
	(m m)
	(mm m)
	(t t)
	(tt t)
	(d d)
	(dd d)
	(c th)
	(s s)
	(ss s)
	(z z)
	(zz z)
	(n n)
	(nn n)
	(l l)
	(ll l)
	(sh sh )
	(sch s ch)
	(zh zh)
	(zhz zh)
	(ch ch)
	(r r)
	(rr r)
	(ij ih)
	(k k)
	(kk k)
	(g g)
	(gg g)
	(h hh)
	(hh hh)
	(# pau)))


(define (Diphone_Prompt_Word utt)
  "(Diphone_Prompt_Word utt)
Specify specific modifications of the utterance before synthesis
specific to this particular phone set."
  (mapcar
   (lambda (s)
     (let ((n (item.name s))
	   (newn (cdr (assoc_string (item.name s) russian2radio_map))))
       (cond
	((cdr newn)  ;; its a dual one
	 (let ((newi (item.insert s (list (car (cdr newn))) 'after)))
	   (item.set_feat newi "end" (item.feat s "end"))
	   (item.set_feat s "end"
			  (/ (+ (item.feat s "segment_start")
				(item.feat s "end"))
			     2))
	   (item.set_name s (car newn))))
	(newn
	 (item.set_name s (car newn)))
	(t
	 ;; as is
	 ))))
   (utt.relation.items utt 'Segment))
  utt)


(provide 'ru_schema)
